﻿using Correo.Abstractions;
using Correo.Application.Extensions;
using Correo.Domain.Models;
using MediatR;
using Microsoft.Extensions.Options;
using System.Threading;
using System.Threading.Tasks;

namespace Correo.Application.CommandHandlers
{
    public class SendEmailHandlerSync
    {
        public record Command : EmailMessage, IRequest<Unit>
        {
        }

        public class CommandHandler : IRequestHandler<Command, Unit>
        {
            private readonly IOptions<DefaultSender> _defaultSender;
            private readonly IMailer _mailer;

            public CommandHandler(IOptions<DefaultSender> defaultSender, IMailer mailer)
            {
                _defaultSender=defaultSender;
                _mailer=mailer;
            }

            public async Task<Unit> Handle(Command command, CancellationToken cancellationToken)
            {
                command.Enrich(_defaultSender.Value);
                command.Validate();
                await _mailer.SendEmailAsync(command, cancellationToken);
                return Unit.Value;
            }
        }
    }
}
