﻿using Correo.Abstractions;
using SendGrid.Helpers.Mail;
using System.Collections.Generic;
using System.Linq;

namespace Correo.SendGrid.Extensions
{
    internal static class ModelExtensions
    {
        private static EmailAddress ToEmailAddress(this EmailMessage.MailAddress address)
            => new EmailAddress(address.Address, address.DisplayName);

        private static List<EmailAddress> ToEmailAddressList(this IEnumerable<EmailMessage.MailAddress> addresses)
            => addresses.Select(z => new EmailAddress(z.Address, z.DisplayName)).ToList();

        public static SendGridMessage ToSendGridMessage(this EmailMessage message)
        {
            var subject = message.Subject;
            var plainContent = !message.IsBodyHtml ? message.Body : null;
            var htmlContent = message.IsBodyHtml ? message.Body : null;
            var from = message.From.ToEmailAddress();
            var to = message.To.First().ToEmailAddress();
            var mailMessage = MailHelper.CreateSingleEmail(from, to, subject, plainContent, htmlContent);

            if (message.To.Count() > 1)
            {
                var tos = message.To.Skip(1).Select(z => new EmailAddress(z.Address, z.DisplayName)).ToList();
                mailMessage.AddTos(tos);
            }

            if (message.Cc != null && message.Cc.Any())
                mailMessage.AddCcs(message.Cc.ToEmailAddressList());

            if (message.Bcc != null && message.Bcc.Any())
                mailMessage.AddBccs(message.Bcc.ToEmailAddressList());

            return mailMessage;
        }
    }
}
